
// (c) COPYRIGHT URI/MIT 1997
// Please read the full copyright statement in the file COPYRIGH.  
//
// Authors:
//	jhrg,jimg	James Gallagher (jgallagher@gso.uri.edu)

// This file contains functions (yes, functions) that process a DAS object
// and create a data stream (either binary or ASCII) which loaddods and
// programs like it can read. I wrote this as a function because the
// structure of the DAS and AttrTable objects makes subclassing them very
// cumbersome. Someday, it might be the case that attributes will become
// reorganized and then I'll change this implementation to a more OO
// approach. jhrg 8/15/97

// $Log: attributes.cc,v $
// Revision 1.4  1999/04/30 17:06:56  jimg
// Merged with no-gnu and release-2-24
//
// Revision 1.3  1997/09/10 22:46:59  jimg
// Fixed the const-ness of a temp in print_attr_values.
//
// Revision 1.2  1997/09/05 16:56:42  jimg
// Added print_select_attributes() which is used to print only those attributes
// that are in a given container.
//
// Revision 1.1  1997/08/18 23:44:00  jimg
// Created to handle output of attribute values for writeval. The attributes
// are written to the ostream passed into print_attributes() using the same
// general form as for variables. Thus, a program like loaddods can decode
// the attributes and do something sensible like intern values.
//

#include "config_dap.h"

#include <iostream>

#include "DAS.h"

static char not_used rcsid[]={"$Id: attributes.cc,v 1.4 1999/04/30 17:06:56 jimg Exp $"};

void print_attributes(DAS &das, const string &dataset_name, ostream &os);
static void print_attr_table(AttrTable *at, const string &attr_name, 
			     ostream &os);
static void print_attr_value(AttrTable *at, Pix p, ostream &os);

/** Scan the DAS object and print the container matching #name# to stdout
  using the same syntax as with the Structure data type.
  @return: True if the container was found, false otherwise. */

bool
print_select_attributes(DAS &das, const string &name, ostream &os)
{
    for(Pix p = das.first_var(); p; das.next_var(p)) {
	const string &n = das.get_name(p);
	if (n == name)
	    print_attr_table(das.get_table(p), n, os);
    }
}

/** Scan the DAS object and write it to stdout using the same `syntax' as
  with the Structure data type. */

void
print_attributes(DAS &das, const string &dataset_name, ostream &os)
{
    os << "Structure" << endl;

    int length = 0;
    for(Pix p = das.first_var(); p; das.next_var(p))
	length++;

    os << dataset_name << " " << length << endl;

    for(Pix p = das.first_var(); p; das.next_var(p)) {
	print_attr_table(das.get_table(p), das.get_name(p), os);
    }
}

/** Print an attribute table. Use the same syntax as with data for the
  matching cardinal data types. For attribute vectors, treat them the same as
  data arrays. */

static void
print_attr_table(AttrTable *at, const string &attr_name, ostream &os)
{
    os << "Structure" << endl;
    
    int length = 0;
    for (Pix p = at->first_attr(); p; at->next_attr(p))
	length++;

    os << attr_name << " " << length << endl;

    for(Pix p = at->first_attr(); p; at->next_attr(p)) {
	if (at->is_container(p))
	    print_attr_table(at->get_attr_table(p), at->get_name(p), os);
	else 
	    print_attr_value(at, p, os);
    }
}

/** Print an attribute value on #os#. All attribute values are assumed to be
  cardinal types (or vectors of a single cardinal type). Attribute
  containers must be handled by print_attr_table(). For attributes whose
  values are vectors, use writeval's Array output format. */

static void
print_attr_value(AttrTable *at, Pix p, ostream &os)
{
    int attr_num = at->get_attr_num(p);
    if (attr_num > 1) {
	// Attribute vector; write as array with one dimension
	os << "Array" << endl;
	os << at->get_type(p) << " " << at->get_name(p) << "1"
	   << at->get_attr_num(p) << endl;
	for (int i = 0; i < attr_num; ++i)
	    os << at->get_attr(p, i) << endl;
    }
    else {
	os << at->get_type(p) << endl;
	os << at->get_name(p) << endl;
	os << at->get_attr(p) << endl;
    }
}
